using System;
using System.Collections.Generic;
using KeysightSD1;

namespace Keysight.Tse.ProgrammingExamples.ProgExample5
{
    /// <summary>
    /// Application Configuration
    /// Defines module descriptors, configuration options and names of HVI engines, actions, triggers
    /// </summary>
    public class ApplicationConfig
    {
        #region Global Options
        /// <summary>
        /// Define PXI instruments options to open the instruments. 
        /// Complete option list can be found in the SD1 3.x User Guide.
        /// </summary>
        public static string Options { get; set; } = "channelNumbering=keysight";

        /// <summary>
        /// Simulated HW Option
        /// </summary>
        public static bool HardwareSimulated { get; set; } = true;

        /// <summary>
        /// Number of loops
        /// </summary>
        public int NumLoops { get; set; } = 5;
        #endregion

        #region Define names of HVI engines, actions, triggers
        /// <summary>
        /// AWG 
        /// </summary>
        public static string AwgEngineName { get; } = "AwgHviEngine";

        /// <summary>
        /// Digital IO Engine Name
        /// </summary>
        public static string DioEngineName { get; } = "DioHviEngine";

        /// <summary>
        /// User-defined name of the leader HVI engine
        /// </summary>
        public static string LeaderEngineName { get; set; } = string.Format("{0}0", AwgEngineName);

        /// <summary>
        /// User-defined HVI trigger name
        /// </summary>
        public static string FpTriggerName { get; } = "FPTrigger";

        /// <summary>
        /// User-defined HVI action name: AWG trigger
        /// </summary>
        public static string AwgTriggerActionName { get; } = "AWGTrigger";

        /// <summary>
        /// User-defined HVI action name: DAQ trigger
        /// </summary>
        public static string DaqTriggerActionName { get; } = "DaqTrigger";
        #endregion

        #region Define HW Platform
        /// <summary>
        /// Define module descriptors below with your instruments information
        /// </summary>
        public static List<ModuleDescriptor> ModuleDescriptors { get; set; } = new List<ModuleDescriptor>
        {
            new ModuleDescriptor{ ApplicationRole = DioEngineName, ModelNumber = "M3202A", ChassisNumber = 2, SlotNumber = 4, Options = Options },
            new ModuleDescriptor{ ApplicationRole = AwgEngineName, ModelNumber = "M3202A", ChassisNumber = 2, SlotNumber = 5, Options = Options },
        };

        /// <summary>
        /// Define list of chassis containing instruments to be used by HVI
        /// </summary>
        public int[] ChassisList { get; set; } = new int[] { 2 };

        /// Multi-chassis setup
        /// Define the System Sync Modules included in your system.
        /// </summary>
        public static string SsmOptions { get; set; } = "";
        public static string SsmSimulationOptions { get; set; } = "Simulate=true,DriverSetup=Model=M9033A";
        public List<SystemSyncModuleDescriptor> SystemSyncModulesDescriptors { get; set; } = new List<SystemSyncModuleDescriptor>
        {
            // new SystemSyncModuleDescriptor("PXI0::CHASSIS1::SLOT10::INDEX0::INSTR", SsmOptions),
        };
        /// </summary>
        /// For each SSM define which SSM is connected to its downstream connectors.
        /// /// Each connectivity item is a triple (ssm1_chassis, ssm1_downstream_connector_number, ssm2_chassis)
        /// </summary>
        public List<SystemSyncModuleConnection> SsmConnections { get; set; } = new List<SystemSyncModuleConnection>
        {
            // new SystemSyncModuleConnection{Ssm1Chassis = 1, Ssm1DownstreamConnectorNumber = 1, Ssm2Chassis = 2},
        };

        /// <summary>
        /// Define PXI Trigger Resources
        /// Assign triggers to HVI object to be used for HVI-managed synchronization, data sharing, etc
        /// </summary>
        public TriggerResourceId[] PxiSyncTriggerResources { get; set; } = new TriggerResourceId[]{
            TriggerResourceId.PxiTrigger0,
            TriggerResourceId.PxiTrigger1,
            TriggerResourceId.PxiTrigger2,
            TriggerResourceId.PxiTrigger3,
            TriggerResourceId.PxiTrigger4
    };
        #endregion

        #region AWG Parameters
        // NOTE: AWG Parameters and functionalities are documented in the M32xxA AWG User Guide

        /// <summary>
        /// Path of the waveform to be queued and played by the AWG channels
        /// </summary>
        public string WaveformPath { get; set; } = "..\\..\\..\\..\\Waveforms\\Gaussian.csv";

        /// <summary>
        /// Sync Mode setting for all AWG channels
        /// </summary>
        public int SyncMode { get; set; } = KeysightSD1.SD_SyncModes.SYNC_NONE;

        /// <summary>
        /// Queue Mode setting for all AWG channels
        /// </summary>
        public int QueueMode { get; set; } = SD_QueueMode.ONE_SHOT;

        /// <summary>
        /// Start Delay value for the waveform queued and played by all AWG channels
        /// </summary>
        public int StartDelay { get; set; } = 0;

        /// <summary>
        /// Prescaler value applied to all the waveform queued and played by all AWG channels
        /// </summary>
        public int AwgPrescaler { get; set; } = 0;

        /// <summary>
        /// Number of cycles setting for the waveform queued and played by all AWG channels
        /// </summary>
        public int AwgNumCycles { get; set; } = 0;

        /// <summary>
        /// Amplitude setting for all AWG channels
        /// </summary>
        public int Amplitude { get; set; } = 1;

        /// <summary>
        /// Trigger mode for all AWG channels
        /// </summary>
        public int AwgTriggerMode { get; set; } = SD_TriggerModes.SWHVITRIG_CYCLE;

        /// <summary>
        /// Mask defining the AWG channels to be 
        /// </summary>
        public int ChannelMask { get; set; } = 0xF;
        #endregion
    }

}
